@@ -14,12 +14,14 @@
 # limitations under the License.
 #
 
-from .common import CertificateRequest, CommonConnection, RevocationRequest, ZoneConfig, CertField, KeyType, CustomField
+from .common import CertificateRequest, CommonConnection, RevocationRequest, ZoneConfig, CertField, KeyType, \
+    CustomField, Authentication, SCOPE_CM, SCOPE_PM, SCOPE_SSH
 from .connection_cloud import CloudConnection
 from .connection_tpp import TPPConnection
 from .connection_tpp_token import TPPTokenConnection
 from .connection_fake import FakeConnection
 from .pem import Certificate
+from .ssh_utils import SSHCertRequest, SSHKeyPair, write_ssh_files
 
 
 def Connection(url=None, token=None, user=None, password=None, fake=False, http_request_kwargs=None):
